@@ -46,7 +46,7 @@ DrawSoundWidget::DrawSoundWidget(QWidget *parent, const char *name,SoundManager*
                 0,20,0,canvas->height()-20,canvas,TRUE);
         selection->hide();
 
-        //verticalSelections.setAutoDelete(TRUE);
+        lockedBeatRects.setAutoDelete(TRUE);
 
         playline=new PlayLine(canvas,0);
 	QPen* pen =new QPen();
@@ -135,20 +135,24 @@ DrawSoundWidget::DrawSoundWidget(QWidget *parent, const char *name,SoundManager*
                                     lockedBeatLines.remove(beatline);
                                     if(lockedBeatLines.count()==0)
                                         emit(hasLockedBeatLines(FALSE));
+
                             }else{
                                     beatline->lock();
                                     lockedBeatLines.append(beatline);
                                     if(lockedBeatLines.count()==1)
                                         emit(hasLockedBeatLines(TRUE));
-
                                     if(beatline->isBpmLine())
                                             bpmLines.remove(beatline);
                                     else
                                             beatLines.remove(beatline);
                             }
                             updateBeatLines();
+                            updateLockedBeatRects();
                             emit(beatLinesChanged(toArray(beatLines)));
                             emit(lockedBeatLinesChanged(toArray(lockedBeatLines)));
+                }else if(LockedBeatRect* beatrect=getMovableBeatRect(mouseEvent->pos())){
+                    beatrect->setSelected(TRUE);
+                    canvas->update();
                 }else{
                     QCanvasItemList colisions=canvas->collisions(pos);
                     /*
@@ -249,6 +253,7 @@ DrawSoundWidget::DrawSoundWidget(QWidget *parent, const char *name,SoundManager*
         handleMouseMoveSelectionRectangles(mouseEvent);
         handleMouseMoveVerticalSelectionRectangles(mouseEvent);
         handleMouseMoveLoopMarkers(mouseEvent);
+        handleMouseMoveBeatRect(mouseEvent);
     }
  void DrawSoundWidget::contentsMouseReleaseEvent( QMouseEvent * mouseEvent){
      scrollTimer->stop();
@@ -287,6 +292,7 @@ DrawSoundWidget::DrawSoundWidget(QWidget *parent, const char *name,SoundManager*
         canvas->update();
     }
     if(BeatLine* beatline=getMovingBeatLine()) beatline->setMoving(FALSE);
+    if(LockedBeatRect* beatrect=getMovingBeatRect()) beatrect->setSelected(FALSE);
     setCursor(QCursor(Qt::Qt::ArrowCursor ));
     canvas->update();
  }
@@ -298,6 +304,7 @@ void DrawSoundWidget::resized(){
     updateLoopMarkers();
     updateSelectionRectangles();
     updateBeatLines();
+    updateLockedBeatRects();
     updateCanvas=true;
     canvas->update();
 }
@@ -393,6 +400,33 @@ void DrawSoundWidget::addBeatLine(long beat){
 
 }
 
+void DrawSoundWidget::addLockedBeatLine(long beat){
+    //locks the beatline if already present
+    bool alreadyPresent=FALSE;
+    QPtrListIterator<BeatLine> it( beatLines );
+    BeatLine *beatline;
+    while ( (beatline = it.current()) != 0 ) {
+        ++it;
+        if(abs(beatline->getBeatPos()-beat)<16){
+            beatline->lock();
+            lockedBeatLines.append(beatline);
+            beatLines.remove(beatline);
+            alreadyPresent=TRUE;
+            break;
+        }
+    }
+    if(!alreadyPresent){
+        BeatLine* beatline=new BeatLine(canvas,beat);
+        beatline->setSampleValueView(sampleValueView);
+        beatline->setPen(QColor(50,255,50));
+        beatline->setX((int)((double)beat/getZoomRatio()));
+        beatline->lock();
+        if(showBeatLinesVal) beatline->show();
+        lockedBeatLines.append(beatline);
+    }
+
+}
+
 
 
 /*!
@@ -733,6 +767,7 @@ void DrawSoundWidget::computeNewBPMLines(const QString & bpm_s){
 	float newBPM=bpm_s.toFloat()*soundManager->getRate()*60;
 	float oldBPM=getUsedBPM()*soundManager->getRate()*60;
 	new_to_oldBPM_ratio=newBPM/oldBPM;
+        /*
 	for (uint i=0;i<newBeatLines.count();i++){
 		delete newBeatLines.at(i);
     		newBeatLines.remove(i);
@@ -745,6 +780,7 @@ void DrawSoundWidget::computeNewBPMLines(const QString & bpm_s){
 		newBeatLines.append(beatline);
 	}
 	updateNewBeatLines();
+        */
 }
 
 
@@ -766,6 +802,7 @@ void DrawSoundWidget::unlockBeatLines()
 		lockedBeatLines.remove(beatline);
     	}
     	updateBeatLines();
+        updateLockedBeatRects();
         emit(beatLinesChanged(toArray(beatLines)));
         emit(lockedBeatLinesChanged(toArray(lockedBeatLines)));
 }
@@ -784,6 +821,7 @@ void DrawSoundWidget::lockBeatLines()
         beatLines.remove(beatline);
     }
     updateBeatLines();
+    updateLockedBeatRects();
     emit(beatLinesChanged(toArray(beatLines)));
     emit(lockedBeatLinesChanged(toArray(lockedBeatLines)));
 }
@@ -985,6 +1023,7 @@ void DrawSoundWidget::zoomChanged(){
     updateCanvas=false;
     setPositionPointer(playPosition);
     updateBeatLines();
+    updateLockedBeatRects();
     updateSelectionRectangles();
     updateVerticalSelectionRectangles();
     updateLoopMarkers();
@@ -1064,21 +1103,6 @@ long DrawSoundWidget::getSelectionEndSample()
 }
 
 
-/*!
-    \fn DrawSoundWidget::getLockedBeats()
- */
-void DrawSoundWidget::getLockedBeats(long tempLocked[])
-{
-
-    QPtrListIterator<BeatLine> it( lockedBeatLines );
-    BeatLine *beatline;
-    int count=0;
-    while ( (beatline = it.current()) != 0 ) {
-        ++it;
-        tempLocked[count]=beatline->getBeatPos();
-        count++;
-    }
-}
 
 /*!
     \fn DrawSoundWidget::getLockedBeats()
@@ -1521,6 +1545,34 @@ BeatLine* DrawSoundWidget::getMovableBeatLine(QPoint pos)
     return 0;
 }
 
+/*!
+        \fn DrawSoundWidget::getMovableBeatRect(QPoint pos)
+ */
+LockedBeatRect* DrawSoundWidget::getMovableBeatRect(QPoint pos)
+{
+    QPtrListIterator<LockedBeatRect> it( lockedBeatRects );
+    LockedBeatRect *beatrect;
+    while ( (beatrect = it.current()) != 0 ) {
+        ++it;
+        if(beatrect->getHandle()->contains(pos)) return beatrect;
+    }
+    return 0;
+}
+
+/*!
+        \fn DrawSoundWidget::getMovingBeatRect()
+ */
+LockedBeatRect* DrawSoundWidget::getMovingBeatRect()
+{
+    QPtrListIterator<LockedBeatRect> it( lockedBeatRects );
+    LockedBeatRect *beatrect;
+    while ( (beatrect = it.current()) != 0 ) {
+        ++it;
+        if(beatrect->isSelected()) return beatrect;
+    }
+    return 0;
+}
+
 /*!
     \fn DrawSoundWidget::getMovableBeatLine(QPoint pos)
  */
@@ -1596,21 +1648,93 @@ void DrawSoundWidget::handleMouseMoveBeatLine(QMouseEvent* mouseEvent)
     }
 }
 
-void DrawSoundWidget::dragEnterEvent(QDragEnterEvent* event)
+/*!
+    \fn DrawSoundWidget::handleMousMoveBeatRect(MouseEvent* mouseEvent)
+ */
+void DrawSoundWidget::handleMouseMoveBeatRect(QMouseEvent* mouseEvent)
 {
-    event->accept(
-            QTextDrag::canDecode(event));
+    if(LockedBeatRect* beatrect=getMovingBeatRect()){
+        QString tempfilename=QString::number(beatrect->getNb()).rightJustify(8,'0').append(".")
+                                                .append(soundManager->getFileExtension());
+
+        QString orig=tempfilename;
+        orig.prepend("orig-").prepend(soundManager->getTempDir()+"/");
+
+        QString final=tempfilename;
+        final.prepend(soundManager->getTempDir()+"/");
+
+        QString ladspa=tempfilename;
+        ladspa.prepend("ladspa-").prepend(soundManager->getTempDir()+"/");
+
+
+        QStrList filelist;
+
+        soundManager->saveChunkToAs(   final,
+                                       beatrect->getStartSample(),
+                                       beatrect->getEndSample(),-1,0,SoundHolder::FOR_OUTPUT);
+        filelist.append(final);
+
+        /*
+        soundManager->saveChunkToAs(   orig,
+                                       beatrect->getStartSample(),
+                                       beatrect->getEndSample(),-1,0,SoundHolder::ORIGINAL_WAVE);
+        filelist.append(orig);
+
+        if(soundManager->getHasLADSPA()){
+            soundManager->saveChunkToAs(   ladspa,
+                                           beatrect->getStartSample(),
+                                           beatrect->getEndSample(),-1,0,SoundHolder::FOR_OUTPUT);
+            filelist.append(ladspa);
+        }
+        */
+
+        WaveDrag* waveDrag=new WaveDrag(filelist,this);
+
+        QPixmap pixmap=QPixmap::grabWidget(this);
+        QPixmap pixmap2;
+        pixmap2.resize((beatrect->getEndSample()-beatrect->getStartSample())/getZoomRatio(),
+                                        pixmap.height()-40);
+        copyBlt(&pixmap2,0,0,&pixmap,beatrect->x()-contentsX(),20,pixmap2.width(),pixmap2.height());
+
+        QWMatrix m;
+        m.scale(.5,.25);
+        QPixmap scaledPixmap = pixmap2.xForm(m);
+        waveDrag->setPixmap(scaledPixmap);
+        waveDrag->dragMove();
+        beatrect->setSelected(FALSE);
+        canvas->update();
+    }
 }
 
-void DrawSoundWidget::dropEvent(QDropEvent* event)
+void DrawSoundWidget::contentsDragEnterEvent(QDragEnterEvent* event)
 {
-    QString text;
-
-    if ( QTextDrag::decode(event, text) ) {
-        emit(receivedEnvelope(event->pos(),text));
+    cout<<"contentsDragEnterEvent\n";
+    if(lockedBeatLines.count()>0 && QUriDrag::canDecode(event)){
+        QStringList list;
+        QUriDrag::decodeLocalFiles(event,list);
+        cout<<"Dragged file: "<<list.front()<<endl;
+        event->accept(soundManager->canOpen(list.front()));
     }
 }
 
+void DrawSoundWidget::contentsDragMoveEvent(QDragMoveEvent* event)
+{
+    if(event->isActionAccepted()){
+        QPoint pos=event->pos();
+        QCanvasItemList colisions=canvas->collisions(pos);
+        cout<<"drag pos: "<<pos.x()<<endl;
+    }
+}
+
+void DrawSoundWidget::contentsDropEvent(QDropEvent* event)
+{
+        int insertpos=event->pos().x();
+        QStringList list;
+        QUriDrag::decodeLocalFiles(event,list);
+        soundManager->insertFileAt(list.front(),0);
+        cout<<"wavefile dropped\n";
+}
+
 
 
 
@@ -1690,3 +1814,106 @@ QValueList<long> DrawSoundWidget::toArray(QPtrList<BeatLine> beatlines)
     qHeapSort(returnArray);
     return returnArray;
 }
+
+
+/*!
+    \fn DrawSoundWidget::findNextLockedBeatLine(BeatLine bl)
+ */
+BeatLine* DrawSoundWidget::findNextLockedBeatLine(BeatLine* bl)
+{
+    lockedBeatLines.sort();
+    QPtrListIterator<BeatLine> it( lockedBeatLines );
+    BeatLine *beatline;
+    while ( (beatline = it.current()) != 0 ) {
+        ++it;
+        if (beatline->getBeatPos()>bl->getBeatPos() && beatline->locked()) return beatline;
+    }
+    return 0;
+}
+
+
+/*!
+    \fn DrawSoundWidget::getLockedBeatList()
+ */
+QValueList<long> DrawSoundWidget::getLockedBeatList()
+{
+    return toArray(lockedBeatLines);
+}
+
+
+/*!
+    \fn DrawSoundWidget::updateLockedBeatRects()
+ */
+void DrawSoundWidget::updateLockedBeatRects()
+{
+
+    lockedBeatLines.sort();
+    lockedBeatRects.clear();
+    QPtrListIterator<BeatLine> it( lockedBeatLines );
+    BeatLine *beatline,*beatline2;
+    int color=0;
+    int count=0;
+    while ( (beatline = it.current()) != 0 ) {
+        ++it;
+        beatline2=it.current();
+        if(beatline2!=0){
+            LockedBeatRect* lr=new LockedBeatRect(canvas,
+                                                    beatline,beatline2);
+            lr->setNb(count);
+            lr->setColor(color);
+            lr->show();
+            lockedBeatRects.append(lr);
+            color=(color+1)%2;
+            count++;
+        }
+    }
+    if(updateCanvas) canvas->update();
+
+
+}
+
+
+/*!
+    \fn DrawSoundWidget::setLockedBeatLines(QValueList <long> lbl)
+ */
+void DrawSoundWidget::setLockedBeatLines(QValueList <long> lbl)
+{
+    for (int i=0;i<lbl.count();i++){
+        addLockedBeatLine(lbl[i]);
+    }
+    updateCanvas=FALSE;
+    updateBeatLines();
+    updateLockedBeatRects();
+    updateCanvas=TRUE;
+    canvas->update();
+    emit(lockedBeatLinesChanged(toArray(lockedBeatLines)));
+}
+
+
+/*!
+    \fn DrawSoundWidget::getLeftLoopMarker()
+ */
+LoopMarker* DrawSoundWidget::getLeftLoopMarker()
+{
+    return loopMarkerL;
+}
+
+
+/*!
+    \fn DrawSoundWidget::getRightLoopMarker()
+ */
+LoopMarker* DrawSoundWidget::getRightLoopMarker()
+{
+    return loopMarkerR;
+}
+
+
+/*!
+    \fn DrawSoundWidget::getBeatList()
+ */
+QValueList <long> DrawSoundWidget::getBeatList()
+{
+    QValueList<long> result=toArray(beatLines)+toArray(lockedBeatLines);
+    qHeapSort(result);
+    return result;
+}
