@@ -697,6 +697,7 @@ void SoundManager::resample(list<long>sortedPos,double new_to_oldBPM_ratio)
 	for (it=sortedPos.begin();it!=sortedPos.end();++it){
 		startpos=endpos;
 		endpos=(int)*it;
+                cout<<"resampling from: "<<startpos<<" to: "<<endpos<<endl;
 		input_buffer_size=endpos-startpos;
 		output_buffer_size=(int)ceil((endpos-startpos)*(1/new_to_oldBPM_ratio));
 
@@ -760,7 +761,12 @@ void SoundManager::resample(list<long>sortedPos,double new_to_oldBPM_ratio)
 
         zaparr(tmp_wave2);
         zaparr(outputWave);
-
+        if(getHasLADSPA()) initBuffer(AFTER_LADSPA,getFrames());
+        if(getHasEnvelope()) initBuffer(AFTER_ENVELOPE,getFrames());
+        getEffects()->setNbFrames(getFrames());
+        emit(reinitLADSPAValues());
+        if(getEffects()->getCurrent()) applyPlugin(getEffects()->getCurrent()); //reapply current plugin
+        applyEnvelope();
         zapInstantEBuffer();
         emit(waveChanged());
 }
@@ -949,6 +955,7 @@ void SoundManager::applyPlugin(LADSPAEffect* effect,long start_sample, long end_
 
     QProgressDialog progress("Applying pugin...","Abort",getFrames(),NULL,"progress",TRUE);
     progress.setMinimumDuration(1000);
+    progress.setProgress(0);
 
     //audio ports
 
@@ -976,7 +983,7 @@ void SoundManager::applyPlugin(LADSPAEffect* effect,long start_sample, long end_
                 processedSamples+=lastChunk;
                 effect->advancePortValues(lastChunk);
             }
-            if((i/1024)%100==0){
+            if(i%(102400)==0){
                 progress.setProgress(i);
                 progress.setLabelText("Applying pugin...(frame: "+QString::number(i)+")");
             }
@@ -985,11 +992,11 @@ void SoundManager::applyPlugin(LADSPAEffect* effect,long start_sample, long end_
             }
             assert(processedSamples<=getFrames());
     }
+    progress.close();
     if(hasEnvelope){
         applyEnvelope();
     }else{
         this->zapInstantEBuffer();
-        //emit(waveChanged(start_sample,end_sample));  //should be..
         emit(waveChanged()); //is ..
     }
 }
@@ -997,7 +1004,6 @@ void SoundManager::applyPlugin(LADSPAEffect* effect,long start_sample, long end_
 
 void SoundManager::applyPlugin(LADSPAEffect* effect){
     applyPlugin(effect,0,getFrames());
-    this->zapInstantEBuffer();
     emit(waveChanged());
 }
 
@@ -1024,7 +1030,7 @@ LADSPAEffects* SoundManager::getEffects()
  */
 void SoundManager::zapInstantEBuffer()
 {
-    cout<<"Zapping buffers\n";
+    cout<<"Zapping Instant Energy buffers\n";
     if(instant_buffer) zaparr(instant_buffer);
     if(hirestemp_subband_energies) zaparr(hirestemp_subband_energies);
     if(hiresfreq_subband_energies) zaparr(hiresfreq_subband_energies);
@@ -1045,9 +1051,11 @@ float** SoundManager::compute_subband_energies(int subband_number,
                                                                                                     bufferType type)
 {
 
+    cout<<"Computing fft\n";
     assert(start_sample>=0 && end_sample<=getFrames());
     QProgressDialog progress("Computing FFT...","Abort",end_sample-start_sample,NULL,"progress",TRUE);
     progress.setMinimumDuration(2000);
+    progress.setProgress(0);
 
     //convert start/end frequency to fft index
     float start_index=(startfreq*frames)/getRate();
@@ -1144,11 +1152,9 @@ float** SoundManager::compute_subband_energies(int subband_number,
                 //end update minimas and maximas
             }
             //end compute the energy of n subbands
-            if(i%5000==0){
-                progress.setProgress(i);
+            if((i-start_sample)%(advance_frames*100)==0){
+                progress.setProgress(i-start_sample);
                 progress.setLabelText("Computing FFT...(frame: "+QString::number(i)+")");
-                QApplication::eventLoop()->processEvents(QEventLoop::AllEvents);
-                QApplication::flushX();
             }
     }
     if(gaussian) zaparr(gaussian);
@@ -1302,9 +1308,9 @@ void SoundManager::recompute_subband_energies(){
 /*!
     \fn SoundManager::setFFTParams(int,int,int,int,int,in,int,int,bool)
  */
-                void SoundManager::setFFTParams(int hf_w,int hf_a,int hf_nb,int hf_p,int hf_sf,int hf_ef,
-                int ht_w,int ht_a,int ht_nb, int ht_p,int ht_sf,int ht_ef,
-                int w, bool ssm)
+void SoundManager::setFFTParams(int hf_w,int hf_a,int hf_nb,int hf_p,int hf_sf,int hf_ef,
+int ht_w,int ht_a,int ht_nb, int ht_p,int ht_sf,int ht_ef,
+int w, bool ssm)
 {
 
 
@@ -1326,7 +1332,9 @@ void SoundManager::recompute_subband_energies(){
 
         substractSpectralMean=ssm;
 
-        recompute_subband_energies();
+        //recompute_subband_energies();
+        //this->zapInstantEBuffer();
+        if(!getHasEnvelope() && !getHasLADSPA()) this->zapInstantEBuffer();
         emit(fftParamsChanged());
         emit(analysisParamsChanged());
 }
@@ -1689,7 +1697,7 @@ void SoundManager::applyEnvelope()
         float* inputBufferRight=SoundHolder::getRightChannel(BEFORE_ENVELOPE);
         float* outputBufferLeft=SoundHolder::getLeftChannel(AFTER_ENVELOPE);
         float* outputBufferRight=SoundHolder::getRightChannel(AFTER_ENVELOPE);
-        if(!envelopeData || !envelopeArray || lockedBeatLines.count()==0){
+        if(!envelopeData || !envelopeArray || lockedBeatLines.count()==0 || !envelopeData->isEnabled()){
                 std::memcpy(outputBufferLeft,inputBufferLeft,sizeof(long)*getFrames());
                 std::memcpy(outputBufferRight,inputBufferRight,sizeof(long)*getFrames());
         }else{
@@ -1769,15 +1777,17 @@ void SoundManager::beatLinesChanged(QValueList<long> beatlines)
 void SoundManager::envelopeChanged(EnvelopeData* envData)
 {
     //compute the envelope
-    envelopeData=envData;
-    long applyNbFrames=(envelopeData->env_duration/1000.0)*float(getRate());
-    if(envelopeArray) zaparr(envelopeArray);
-    envelopeArray=new float[applyNbFrames];
-    for (int i=0;i<applyNbFrames;i++){
-        envelopeArray[i]=envelopeData->getEnvAt(float(i)/float(applyNbFrames),
-                                           100.0/float(applyNbFrames));
+    if(envData){
+        envelopeData=envData;
+        long applyNbFrames=(envelopeData->env_duration/1000.0)*float(getRate());
+        if(envelopeArray) zaparr(envelopeArray);
+        envelopeArray=new float[applyNbFrames];
+        for (int i=0;i<applyNbFrames;i++){
+            envelopeArray[i]=envelopeData->getEnvAt(float(i)/float(applyNbFrames),
+                                            100.0/float(applyNbFrames));
+        }
+        applyEnvelope();
     }
-    applyEnvelope();
 }
 
 
@@ -1797,3 +1807,35 @@ bool SoundManager::getHasEnvelope()
 {
     return hasEnvelope;
 }
+
+
+/*!
+    \fn SoundManager::getFileExtension()
+ */
+QString  SoundManager::getFileExtension()
+{
+    return fileManager->getFilenameExtension();
+}
+
+
+/*!
+    \fn SoundManager::getTempDir()
+ */
+QString SoundManager::getTempDir()
+{
+    return fileManager->getTempDir();
+}
+
+
+/*!
+    \fn SoundManager::canOpen(QString file)
+ */
+bool SoundManager::canOpen(QString file)
+{
+    bool openable=FALSE;
+    SF_INFO tempsfinfo;
+    SNDFILE* tmp_sndfile= sf_open(file, SFM_READ, &tempsfinfo);
+    if(tmp_sndfile==0) openable=FALSE; else openable=TRUE;
+    sf_close(tmp_sndfile);
+    return openable;
+}
